using System;

namespace MyNamespace {

	public class Main {
        private void LoadData() {
            var columns = new string[] { "SNOMED_CID", "SNOMED_FSN", "SNOMED_Cncpt_Stats", "UMLS_CUI", "occurrence", "usage", "firstInSubset", "retiredFromSubset", "lastInSubset", "replacedSNOMED_CID" };
            string d = "|", n = "ZEHRSnomedCoreSub";
            using (Oracle.ManagedDataAccess.Client.OracleConnection cn = new("connectionString"))
            using (Oracle.ManagedDataAccess.Client.OracleBulkCopy bulkcopy = new(cn))
            using (System.Data.IDataReader fdr = new MyNamespace.FileDataReader("data.txt", d, columns, true)) {
                foreach (string column in columns) {
                    bulkcopy.ColumnMappings.Add(column, column);
                }
                bulkcopy.BatchSize = 500;
                bulkcopy.BulkCopyTimeout = 60;
				bulkcopy.DestinationSchemaName = "SchemaName";
                bulkcopy.DestinationTableName = n;
                cn.Open();
                bulkcopy.WriteToServer(fdr);
                cn.Close();
                fdr.Close();
            }
        }
	}

    #region FileDataReader Class
	
    public class FileDataReader : System.Data.IDataReader {

        #region Class variables

        private string[] fields = Array.Empty<string>();
        private System.Collections.Generic.List<FileFieldInfo> fieldsInfo;
        private string delimiter = null;
        private System.IO.StreamReader sr = null;
        private int recordsRead = 0;
        private bool closed = false;

        #endregion

        #region Constructors

        /// <summary>
        /// When using this constructor, it is assumed the delimited file contains a header row with the field names. All fields will be of String data type.
        /// </summary>
        /// <param name="filePath">Full path to file, including file name</param>
        /// <param name="delimiter">Field separator</param>
        public FileDataReader(string filePath, string delimiter) {
            this.delimiter = delimiter;
            this.sr = new System.IO.StreamReader(filePath);
            this.fieldsInfo = new System.Collections.Generic.List<FileFieldInfo>();
            foreach (string s in sr.ReadLine().Split(this.delimiter))
                fieldsInfo.Add(new FileFieldInfo(s, typeof(string)));
        }

        /// <summary>
        /// Initializes a new FileDataReader object with the provided field names for a delimited file. All fields will be of String data type.
        /// </summary>
        /// <param name="filePath">Full path to file, including file name</param>
        /// <param name="delimiter">Field separator</param>
        /// <param name="fieldNames">String array containing the name of each field in the file</param>
        /// <param name="containsHeader">Indicates if the file contains a header row which needs to be ignored</param>
        public FileDataReader(string filePath, string delimiter, string[] fieldNames, bool containsHeader = false) {
            this.delimiter = delimiter;
            this.fieldsInfo = new System.Collections.Generic.List<FileFieldInfo>();
            foreach (string s in fieldNames)
                this.fieldsInfo.Add(new FileFieldInfo(s, typeof(string)));
            this.sr = new System.IO.StreamReader(filePath);
            if (containsHeader)
                sr.ReadLine();
        }

        /// <summary>
        /// Initializes a new FileDataReader object with the provided field information for a delimited file
        /// </summary>
        /// <param name="filePath">Full path to file, including file name</param>
        /// <param name="delimiter">Field separator</param>
        /// <param name="fieldsInfo">String array containing the name of each field in the file</param>
        /// <param name="containsHeader">Indicates if the file contains a header row which needs to be ignored</param>
        public FileDataReader(string filePath, string delimiter, System.Collections.Generic.List<FileFieldInfo> fieldsInfo, bool containsHeader = false) {
            this.delimiter = delimiter;
            this.fieldsInfo = fieldsInfo;
            this.sr = new System.IO.StreamReader(filePath);
            if (containsHeader)
                sr.ReadLine();
        }

        /// <summary>
        /// Initializes a new FileDataReader object with the provided field information for a non-delimited file
        /// </summary>
        /// <param name="filePath">Full path to file, including file name</param>
        /// <param name="fieldsInfo">String array containing the name of each field in the file</param>
        /// <param name="containsHeader">Indicates if the file contains a header row which needs to be ignored</param>
        public FileDataReader(string filePath, System.Collections.Generic.List<FileFieldInfo> fieldsInfo, bool containsHeader = false) {
            this.fieldsInfo = fieldsInfo;
            this.sr = new System.IO.StreamReader(filePath);
            if (containsHeader)
                sr.ReadLine();
        }

        #endregion

        #region IDataReader Support

        public void Close() {
            sr.Close();
            closed = true;
        }

        public int Depth {
            get {
                return 0;
            }
        }

        public System.Data.DataTable GetSchemaTable() {
            System.Data.DataTable dt = new System.Data.DataTable();
            dt.Columns.Add("Name", typeof(string));
            dt.Columns.Add("Type", typeof(Type));
            dt.Columns.Add("Format", typeof(string));
            dt.Columns.Add("Length", typeof(int));
            foreach (FileFieldInfo ffi in fieldsInfo)
                dt.Rows.Add(new object[] { ffi.Name, ffi.Type, ffi.Format, ffi.Length });
            return dt;
        }

        public bool IsClosed {
            get {
                return closed;
            }
        }

        public bool NextResult() {
            return Read();
        }

        public bool Read() {
            if (sr.EndOfStream)
                return false;
            else if (this.delimiter != null) {
                this.fields = sr.ReadLine().Split(this.delimiter);
                for (int i = 0; i <= this.fieldsInfo.Count - 1; i++) {
                    this.fields[i] = this.fields[i].Trim();
                    if (this.fields[i].Length == 0)
                        this.fields[i] = this.fieldsInfo[i].DefaultValue as string;
                }
                recordsRead += 1;
                return true;
            } else {
                if (this.fields.Length == 0)
                    this.fields = new string[this.FieldCount];
                string line = sr.ReadLine();
                for (int i = 0; i <= this.fieldsInfo.Count - 1; i++) {
                    if (this.fieldsInfo[i].Length > 0) {
                        this.fields[i] = line.Substring(0, this.fieldsInfo[i].Length);
                        line = line.Remove(0, this.fieldsInfo[i].Length);
                    } else if (this.fieldsInfo[i].Length == -1) {
                        this.fields[i] = line;
                        break;
                    }
                    this.fields[i] = this.fields[i].Trim();
                    if (this.fields[i].Length == 0)
                        this.fields[i] = this.fieldsInfo[i].DefaultValue as string;
                    i += 1;
                }
                recordsRead += 1;
                return true;
            }
        }

        public int RecordsAffected {
            get {
                return recordsRead;
            }
        }

        public int FieldCount {
            get {
                return fieldsInfo.Count;
            }
        }

        public bool GetBoolean(int i) {
            if (int.TryParse(fields[i], out int j))
                return System.Convert.ToBoolean(j);
            else
                return bool.Parse(fields[i]);
        }

        public byte GetByte(int i) {
            return byte.Parse(fields[i]);
        }

        public long GetBytes(int i, long fieldOffset, byte[] buffer, int bufferoffset, int length) {
            byte[] bytes = System.Text.Encoding.Default.GetBytes(fields[i]);
            var j = fieldOffset;
            var k = bufferoffset;
            var l = 0;
            while (l < length && j < bytes.LongLength && k < buffer.LongLength) {
                buffer[k] = bytes[j];
                j += 1;
                k += 1;
                l += 1;
            }
            return l;
        }

        public char GetChar(int i) {
            return char.Parse(fields[i]);
        }

        public long GetChars(int i, long fieldoffset, char[] buffer, int bufferoffset, int length) {
            char[] chars = fields[i].Substring((int)fieldoffset, length).ToCharArray();
            chars.CopyTo(buffer, bufferoffset);
            return chars.LongLength;
        }

        public System.Data.IDataReader GetData(int i) {
            return this;
        }

        public string GetDataTypeName(int i) {
            return GetFieldType(i).Name;
        }

        public DateTime GetDateTime(int i) {
            if (this.fieldsInfo[i].Format != null)
                return DateTime.ParseExact(this.fields[i], this.fieldsInfo[i].Format, System.Globalization.DateTimeFormatInfo.CurrentInfo, System.Globalization.DateTimeStyles.None);
            else
                return DateTime.Parse(this.fields[i]);
        }

        public decimal GetDecimal(int i) {
            return decimal.Parse(fields[i]);
        }

        public double GetDouble(int i) {
            return double.Parse(fields[i]);
        }

        public Type GetFieldType(int i) {
            return this.fieldsInfo[i].Type;
        }

        public float GetFloat(int i) {
            return float.Parse(fields[i]);
        }

        public Guid GetGuid(int i) {
            return Guid.Parse(fields[i]);
        }

        public short GetInt16(int i) {
            return short.Parse(fields[i]);
        }

        public int GetInt32(int i) {
            return int.Parse(fields[i]);
        }

        public long GetInt64(int i) {
            return long.Parse(fields[i]);
        }

        public string GetName(int i) {
            return this.fieldsInfo[i].Name;
        }

        public int GetOrdinal(string name) {
            return this.fieldsInfo.FindIndex((FileFieldInfo ffi) => ffi.Name == name);
        }

        public string GetString(int i) {
            return fields[i];
        }

        public object GetValue(int i) {
            if (this.fieldsInfo[i].Type == typeof(DateTime))
                return GetDateTime(i);
            else if (this.fieldsInfo[i].Type != typeof(string)) {
                try {
                    return Convert.ChangeType(fields[i], this.fieldsInfo[i].Type);
                } catch (Exception) {
                    return fields[i];
                }
            } else
                return fields[i];
        }

        public int GetValues(object[] values) {
            values = fields;
            return values.Length;
        }

        public bool IsDBNull(int i) {
            return fields[i] == null;
        }

        public object this[int i] {
            get {
                return GetValue(i);
            }
        }

        public object this[string name] {
            get {
                return fields[GetOrdinal(name)];
            }
        }

        #endregion

        #region IDisposable Support

        private bool disposedValue; // To detect redundant calls

        // IDisposable
        protected virtual void Dispose(bool disposing) {
            if (!this.disposedValue) {
                if (disposing) {
                    if (sr != null) {
                        try {
                            if (!closed)
                                sr.Close();
                        } catch {
                        }
                        try {
                            sr.Dispose();
                        } catch {
                        }
                    }
                    sr = null;
                    fields = null;
                    fieldsInfo = null;
                }
            }
            this.disposedValue = true;
        }

        // TODO: override Finalize() only if Dispose(ByVal disposing As Boolean) above has code to free unmanaged resources.
        // Protected Overrides Sub Finalize()
        // ' Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
        // Dispose(False)
        // MyBase.Finalize()
        // End Sub

        // This code added by Visual Basic to correctly implement the disposable pattern.
        public void Dispose() {
            // Do not change this code.  Put cleanup code in Dispose(disposing As Boolean) above.
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        #endregion

    }

    #endregion
	
    #region FileFieldInfo Class

    public class FileFieldInfo {

        #region Constructors

        public FileFieldInfo() : base() {
        }

        /// <summary>Initializes a new instance of the FileFieldInfo class with the provided parameters.</summary>
        /// <param name="name">Field's name.</param>
        /// <param name="type">Field's data type.</param>
        /// <param name="format">Field format mask</param>
        /// <param name="length">Maximum length of field</param>
        /// <param name="defaultValue">Default value for field when value read from file is blank</param>
        /// <remarks>
        /// Length only applies to non-delimited files. 
        /// A length of -1 means read to the end of the line.
        /// Therefore, only the last field can have this variable length.
        /// </remarks>
        public FileFieldInfo(string name, Type type, string format = null, int length = 0, object defaultValue = null) : this() {
            this.Name = name;
            this.Type = type;
            this.Format = format;
            this.Length = length;
            this.DefaultValue = defaultValue;
        }

        /// <summary>Initializes a new instance of the FileFieldInfo class with the provided parameters.</summary>
        /// <param name="name">Field's name.</param>
        /// <param name="type">Field's data type.</param>
        /// <param name="length">Maximum length of field</param>
        /// <param name="defaultValue">Default value for field when value read from file is blank</param>
        /// <remarks>
        /// Length only applies to non-delimited files. 
        /// A length of -1 means read to the end of the line.
        /// Therefore, only the last field can have this variable length.
        /// </remarks>
        public FileFieldInfo(string name, Type type, int length, object defaultValue = null) : this() {
            this.Name = name;
            this.Type = type;
            this.Length = length;
            this.DefaultValue = defaultValue;
        }

        /// <summary>Initializes a new instance of the FileFieldInfo class with the provided <c>Name</c> and <c>Length</c>. The <c>Type</c> is assumed to be <c>String</c>.</summary>
        /// <param name="name">Field's name.</param>
        /// <param name="length">Maximum length of field</param>
        /// <param name="defaultValue">Default value for field when value read from file is blank</param>
        /// <remarks>
        /// Length only applies to non-delimited files. 
        /// A length of -1 means read to the end of the line.
        /// Therefore, only the last field can have this variable length.
        /// </remarks>
        public FileFieldInfo(string name, int length, object defaultValue = null) : this() {
            this.Name = name;
            this.Type = typeof(string);
            this.Length = length;
            this.DefaultValue = defaultValue;
        }

        #endregion

        public string Name { get; set; }

        public Type Type { get; set; }

        public string Format { get; set; }

        public int Length { get; set; }

        public object DefaultValue { get; set; } = null;
    }

    #endregion
	
}
